CGROUP		GROUP	CODE
CODE		SEGMENT BYTE PUBLIC 'CODE'
		ASSUME	CS:CGROUP, DS:CGROUP

;------------------------------------------------------------------------------
; DECLARE PUBLIC PROCEDURES
;------------------------------------------------------------------------------
	PUBLIC	XDV_ABORT
	PUBLIC	XDV_EXPORT
	PUBLIC	XDV_IMPORT
	PUBLIC	XDV_POLL
	PUBLIC	XDV_TERMINATE
	PUBLIC	XDV_SAVE
	PUBLIC	XDV_DETACH
	PUBLIC	XDV_ADJUST
	PUBLIC	XDV_CLEAR
	PUBLIC	XDV_ATTACH
	PUBLIC	XDV_RETRIEVE
	PUBLIC	XDV_MAIN
	PUBLIC	XDV_HELP
	PUBLIC	XDV_UNDO_OKAY
	PUBLIC	XDV_UNDO_SWAP
	PUBLIC	XDV_UNDO_SAVE
	PUBLIC	XDV_SYSINFO
	PUBLIC	FILE_EXT

;------------------------------------------------------------------------------
; DECLARE EXTERNAL PROCEDURES
;------------------------------------------------------------------------------
	EXTRN	DO_ERASE:NEAR
	EXTRN	DO_VIDEOSYNC:NEAR
	EXTRN	DO_ERROR:NEAR
	EXTRN	DO_GETFILE:NEAR
	EXTRN 	DO_RANGE:NEAR
	EXTRN	IN_CELL:NEAR
	EXTRN	DO_WAIT:NEAR	
	EXTRN	DO_ERASE_LINES:NEAR	
	EXTRN	DO_HIDEMARK:NEAR
	EXTRN	DO_SETCUR:NEAR
	EXTRN	DO_WRITE:NEAR	
	EXTRN	GET_CHAR:NEAR
	EXTRN	REDISPLAY:NEAR

;------------------------------------------------------------------------------
; DECLARE EXTERNAL VARIABLES
;------------------------------------------------------------------------------
	EXTRN	FILENAME:BYTE
	EXTRN	START_COL:WORD
	EXTRN	START_ROW:WORD
	EXTRN	END_COL:WORD
	EXTRN	END_ROW:WORD
	EXTRN	ANCHORED:BYTE

;------------------------------------------------------------------------------
; EQUATES 
;------------------------------------------------------------------------------

;-----	BOOLEAN VALUES
TRUE		EQU	1
FALSE		EQU	0

;-----	CELL TYPES
T_LABEL		EQU	1

;-----	DEFAULT CELL FORMAT 
FMT_DEFAULT	EQU	07FH

;-----	LAST ROW IN SPREADSHEET
MAXROW		EQU	8191

;-----	HELP SCREEN 
BOX_BOTTOM	EQU	6	;LINE CHARACTER
AT_NORMAL	EQU	0	;TEXT DISPLAY ATTRIBUTE

;------------------------------------------------------------------------------
; DATA 
;------------------------------------------------------------------------------

;-----	FLAGS
ACTIVE_FLAG	DB	FALSE	
FILE_FLAG	DB	TRUE
BOLD_FLAG	DB	FALSE		;BOLD
UNDER_FLAG	DB	FALSE		;UNDERLINE
SUPER_FLAG	DB	FALSE		;SUPERSCRIPT
SUB_FLAG	DB	FALSE		;SUBSCRIPT

;-----	FILE PARAMETERS
FILE_EXT	DB	'*',0		;FILE EXTENSION
FILE_CHAR	DB	?
FILE_HANDLE  	DW   	?

;-----	LINE BUFFER PARAMETERS
LINE_BUFFER	DB	027H		;PLACE ' AT START OF BUFFER
		DB	241 DUP (?)	;240 CHARS PLUS TRAILING 0

CUR_POINTER 	DW	?		;POINTER TO POSITION WITHIN THE BUFFER

;-----	SPREADSHEET PARAMETERS
CUR_ROW	    	DW	?		;CURRENT SPREADSHEET ROW
CUR_COL	    	DW	?		;CURRENT SPREADSHEET COLUMN

;------------------------------------------------------------------------------
; MESSAGES 
;------------------------------------------------------------------------------

;-----	PROMPTS
FILE_PROMPT	DB	'WordStar file: ',0
RANGE_PROMPT	DB	'Upper left corner of target range: ',0

;-----	ERROR MESSAGES
OPEN_ERR_MSG	DB   	'Cannot find file',0
READ_ERR_MSG	DB	'Error reading file',0
BADFILE_ERR_MSG DB	'Incomplete file',0
ROW_ERR_MSG   	DB	'Cannot go beyond last row in spreadsheet',0

;-----	HELP SCREEN TEXT
HELP_LINE	DB	78 DUP (BOX_BOTTOM),0

HELP1		DB 'Help for WORDSTAR.ADN',0

HELP2		DB 'The WORDSTAR.ADN add-in reads a WordStar file on disk and '
		DB 'converts its',0
HELP3		DB 'contents from WordStar',39,'s special format to 1-2-3',39
		DB 's format.  The program',0
HELP4		DB 'places the contents of the file directly in a 1-2-3 '
		DB 'spreadsheet range.',0
HELP5		DB 'You can then modify the range using 4WORD: The Add-In '
		DB 'Word Processor for',0
HELP6		DB '1-2-3.  When the add-in imports the file, it handles '
		DB 'special enhancements',0
HELP7		DB 'including boldface, underline, superscript and '
		DB 'subscript.',0

HELP8		DB 'When you activate the add-in, the program asks you for '
		DB 'the name of the',0
HELP9		DB 'WordStar file you want to import.  Next, it asks you for '
		DB 'the upper left-hand',0
HELP10		DB 'corner of the target 1-2-3 range.  This is the location '
		DB 'where the add-in',0
HELP11		DB 'will begin writing the converted WordStar text.',0

HELP12		DB 'NOTE: Make sure you clear the target range.  Otherwise, '
		DB 'the program over-',0
HELP13		DB 'writes the existing contents of the range.',0

;------------------------------------------------------------------------------
; NAME:		XDV_MAIN
;
; PURPOSE:	MAIN ENTRY POINT FOR A 1-2-3 DYNAMIC DRIVER
;
; PARAMETERS:	NONE
;
; RETURNS:	NOTHING
;------------------------------------------------------------------------------
XDV_MAIN	PROC	NEAR

;-----	STORE SEGMENT, SET SEGMENT TO NEW

		PUSH	DS
		MOV	AX,CS
		MOV	DS,AX
		MOV	ES,AX

;-----	SET FLAG TO INDICATE ADD-IN IS ACTIVE

		MOV	ACTIVE_FLAG,TRUE

;-----	GET NAME OF WORDSTAR FILE
XM2:
		MOV	AH,0		;GET NEW FILE NAME
		LEA	SI,FILE_PROMPT

		CALL	DO_GETFILE
		JNC	XM1		;ESC PRESSED?

		POP	DS		;YES, RETURN TO 1-2-3
		RET

;-----	GET SPREADSHEET RANGE TO LOCATE THE CONTENTS OF THE FILE
XM1:
		MOV	AH,0		;AH=GET NEW RANGE
		MOV	AL,0		;AL=RANGE NOT INITIALLY ANCHORED
		MOV	CH,0		;CH=HIDDEN COLUMNS DISPLAYED
		LEA	SI,RANGE_PROMPT ;SI=POINTER TO RANGE PROMPT

		CALL	DO_RANGE
		JC	XM2		;IF ESC PRESSED, GET FILE NAME AGAIN

;-----	SAVE CURRENT COLUMN AND ROW

		MOV	AX,START_COL
		MOV	CUR_COL,AX

		MOV	AX,START_ROW
		MOV	CUR_ROW,AX

;-----	INITIALIZE CURRENT POINTER WITHIN LINE_BUFFER

		MOV	AX,OFFSET LINE_BUFFER	;POSITION TO '
		INC	AX			;BUMP TO 1 BEYOND '
		MOV	CUR_POINTER,AX 

;-----	OPEN THE FILE

		CALL 	OPEN_FILE

;-----	PUT UP WAIT INDICATOR

		CALL	DO_WAIT

;-----	READ A BYTE FROM THE FILE
XM3:
                CALL 	READ_CHAR	

;-----	JUMP TO THE APPROPRIATE SECTION FOR EACH CHARACTER

		CMP	FILE_CHAR,1AH	;END OF FILE (CTRL-Z)
		JNZ	NO_EOF	
		JMP	EOF
NO_EOF:

		CMP	FILE_CHAR,8AH	;END OF PAGE
		JNZ	NO_EOP
		JMP	LF		;TREAT LIKE LINE FEED CHARACTER
NO_EOP:	

		CMP	FILE_CHAR,0DH	;END OF PARAGRAPH (CARRIAGE RETURN)
		JNZ	NO_CR
		JMP	CR
NO_CR:

		CMP	FILE_CHAR,8DH	;END OF LINE, BUT NOT END OF PARAGRAPH 
		JNZ	NO_EOL
		JMP	EOL
NO_EOL:

		CMP	FILE_CHAR,0AH	;LINE FEED
		JNZ	NO_LF
		JMP	LF
NO_LF:

		CMP	FILE_CHAR,02H	;BOLD
		JNZ	NO_BOLD
		JMP	BOLD
NO_BOLD:

		CMP	FILE_CHAR,82H	;BOLD (END OF WORD)
		JNZ	NO_BOLD_EOW
		JMP	BOLD
NO_BOLD_EOW:

		CMP	FILE_CHAR,13H	;UNDERLINE 
		JNZ	NO_UNDERLINE
		JMP	UNDERLINE
NO_UNDERLINE:

		CMP	FILE_CHAR,93H	;UNDERLINE (END OF WORD)
		JNZ	NO_UNDERLINE_EOW
		JMP	UNDERLINE
NO_UNDERLINE_EOW:

		CMP	FILE_CHAR,14H	;SUPERSCRIPT 
		JNZ	NO_SUPERSCRIPT
		JMP	SUPERSCRIPT
NO_SUPERSCRIPT:

		CMP	FILE_CHAR,94H	;SUPERSCRIPT (END OF WORD)
		JNZ	NO_SUPERSCRIPT_EOW
		JMP	SUPERSCRIPT
NO_SUPERSCRIPT_EOW:

		CMP	FILE_CHAR,16H	;SUBSCRIPT 
		JNZ	NO_SUBSCRIPT
		JMP	SUBSCRIPT
NO_SUBSCRIPT:

		CMP	FILE_CHAR,96H	;SUBSCRIPT (END OF WORD)
		JNZ	NO_SUBSCRIPT_EOW
		JMP	SUBSCRIPT
NO_SUBSCRIPT_EOW:

		JMP	DEFAULT		;ALL OTHER CHARACTERS

;----	CASE: END OF FILE
EOF:
		MOV	AX,OFFSET LINE_BUFFER+1
		CMP	AX,CUR_POINTER	;CASE: LAST LINE OF FILE WRITTEN?
		JZ	EOF_LEAVE	;YES, GO CLOSE FILE AND LEAVE

		MOV	FILE_CHAR,0	;NO, PUT 0 ON END OF LINE BUFFER
		CALL	CHAR_TO_LINE

		CALL	LINE_TO_CELL	;    PUT LAST LINEFUL OF DATA IN CELL
EOF_LEAVE:

		CALL	CLOSE_FILE	;CLOSE THE FILE	

		POP	DS		;RETURN TO 1-2-3
		RET	

;-----	CASE: END OF PARAGRAPH (CARRIAGE RETURN)
CR:
		MOV	FILE_CHAR,14H	;PUT 4WORD CR ON END OF LINE BUFFER 
		CALL	CHAR_TO_LINE
		JMP	XM3

;-----	CASE: END OF LINE, BUT NOT END OF PARAGRAPH
EOL:
		JMP	XM3		;DO NOTHING WITH THIS CHARACTER

;-----	CASE: LINE FEED
LF:
		MOV	FILE_CHAR,0	;PUT TRAILING 0 ON END OF LINE BUFFER
		CALL	CHAR_TO_LINE

		CALL	LINE_TO_CELL	;PUT LINEFUL OF DATA IN A CELL

		MOV	AX,OFFSET LINE_BUFFER	;RESET BUFFER POINTER
		INC	AX
		MOV	CUR_POINTER,AX
		JMP	XM3

;----	CASE: BEGIN BOLD
BOLD:
		CMP	BOLD_FLAG,TRUE	;ENHANCEMENT IN EFFECT?
		JZ	END_BOLD	;YES, GO END IT

		MOV	BOLD_FLAG,TRUE	;NO, SET FLAG
		MOV	FILE_CHAR,97H	;    PUT BEGIN ATTRIBUTE IN LINE BUFFER
		CALL	CHAR_TO_LINE

		MOV	FILE_CHAR,'B'	;    FOLLOWED BY "B"
		CALL	CHAR_TO_LINE			
		JMP	XM3

END_BOLD:
		MOV	BOLD_FLAG,FALSE ;RESET FLAG
		MOV	FILE_CHAR,98H	;END ATTRIBUTE ON END OF LINE BUFFER
		CALL	CHAR_TO_LINE
		JMP	XM3

;----	CASE: BEGIN UNDERLINE
UNDERLINE:
		CMP	UNDER_FLAG,TRUE	;ENHANCEMENT IN EFFECT?
		JZ	END_UNDERLINE	;YES, GO END IT 

		MOV	UNDER_FLAG,TRUE	;NO, SET FLAG
		MOV	FILE_CHAR,97H	;    PUT BEGIN ATTRIBUTE IN LINE BUFFER
		CALL	CHAR_TO_LINE

		MOV	FILE_CHAR,'U'	;    FOLLOWED BY "U"
		CALL	CHAR_TO_LINE			
		JMP	XM3

END_UNDERLINE:
		MOV	UNDER_FLAG,FALSE ;RESET FLAG
		MOV	FILE_CHAR,98H	;END ATTRIBUTE ON END OF LINE BUFFER
		CALL	CHAR_TO_LINE
		JMP	XM3

;----	CASE: BEGIN SUPERSCRIPT
SUPERSCRIPT:
		CMP	SUPER_FLAG,TRUE	;ENHANCEMENT IN EFFECT?
		JZ	END_SUPERSCRIPT	;YES, GO END IT 

		MOV	SUPER_FLAG,TRUE	;NO, SET FLAG
		MOV	FILE_CHAR,97H	;    PUT BEGIN ATTRIBUTE IN LINE BUFFER
		CALL	CHAR_TO_LINE

		MOV	FILE_CHAR,'+'	;    FOLLOWED BY "+"
		CALL	CHAR_TO_LINE			
		JMP	XM3

END_SUPERSCRIPT:
		MOV	SUPER_FLAG,FALSE ;RESET FLAG
		MOV	FILE_CHAR,98H	;END ATTRIBUTE ON END OF LINE BUFFER
		CALL	CHAR_TO_LINE
		JMP	XM3

;----	CASE: BEGIN SUBSCRIPT
SUBSCRIPT:
		CMP	SUB_FLAG,TRUE	;ENHANCEMENT IN EFFECT?
		JZ	END_SUBSCRIPT	;YES, GO END IT 

		MOV	SUB_FLAG,TRUE	;NO, SET FLAG
		MOV	FILE_CHAR,97H	;    PUT BEGIN ATTRIBUTE IN LINE BUFFER
		CALL	CHAR_TO_LINE

		MOV	FILE_CHAR,'-'	;    FOLLOWED BY "-"
		CALL	CHAR_TO_LINE			
		JMP	XM3

END_SUBSCRIPT:
		MOV	SUB_FLAG,FALSE 	;RESET FLAG
		MOV	FILE_CHAR,98H	;END ATTRIBUTE ON END OF LINE BUFFER
		CALL	CHAR_TO_LINE
		JMP	XM3

;-----	CASE: DEFAULT
DEFAULT:
		MOV	AL,FILE_CHAR
		AND	AL,7FH		;FLIP HIGH-ORDER BIT IF NECESSARY
		MOV	FILE_CHAR,AL

		CALL	CHAR_TO_LINE	;PUT CHARACTER AT END OF LINE BUFFER
		JMP	XM3

XDV_MAIN	ENDP


;------------------------------------------------------------------------------
; NAME:		OPEN_FILE
;
; PURPOSE:	OPEN FILE ON THE CURRENT DIRECTORY.
;		
;		IF THE FILE CANNOT BE OPENED, THEN DO AN ERROR MESSAGE. 
;
;		IF THE FILE IS OPENED, THE FOLLOWING VARIABLE IS SET:
;		 FILE_HANDLE=DOS FILE HANDLE
;------------------------------------------------------------------------------
OPEN_FILE	PROC 	NEAR

;-----	SET DX=POINTER TO FILE NAME

		LEA  	DX,FILENAME

;-----	ATTEMPT TO OPEN THE FILE IN THE CURRENT DIRECTORY

		MOV  	AL,0			;OPEN FOR READING
		MOV  	AH,03DH
		INT	21H

;-----	CHECK: FILE OPENED OKAY?

		JC	OF1			;NO, GO DO FATAL ERROR

		MOV	FILE_HANDLE,AX 		;YES, SAVE THE FILE HANDLE
		MOV	FILE_FLAG,TRUE		;SET FLAG INDICATING OPEN FILE 

		RET				;DONE, RETURN

OF1:

;------------------------------------------------------------------------------
; ERROR: FILE NOT FOUND
;------------------------------------------------------------------------------
	
;-----	SET FLAG TO INDICATE FILE NOT OPEN 

		MOV	FILE_FLAG,FALSE

;-----	DO A FATAL ERROR

		LEA	SI,OPEN_ERR_MSG
		CALL	DO_ERROR

OPEN_FILE	ENDP


;------------------------------------------------------------------------------
; NAME:		READ_CHAR
;
; PURPOSE:	READ BYTE FROM WORDSTAR FILE,
;------------------------------------------------------------------------------
READ_CHAR	PROC 	NEAR

		MOV	BX,FILE_HANDLE		;BX=FILE HANDLE
               	MOV  	CX,1		 	;CX=READ A LENGTH OF 1 BYTE
		LEA	DX,FILE_CHAR		;DX=BUFFER TO PUT BYTE
		MOV  	AH,03FH              	;FILE READ
		INT  	21H

		JC  	RF1                	;CHECK: ERROR READING?

		CMP	AX,0			;CHECK: END OF FILE?
		JZ	RF2                	;GO DO FATAL ERROR

		RET

RF1:
;------------------------------------------------------------------------------
; ERROR: READING FILE 
;------------------------------------------------------------------------------
	
;-----	DO A FATAL ERROR

		LEA	SI,READ_ERR_MSG
		CALL	DO_ERROR

RF2:	
;------------------------------------------------------------------------------
; ERROR: INCOMPLETE FILE (UNLIKELY)
;------------------------------------------------------------------------------

;-----	DO A FATAL ERROR

		LEA	SI,BADFILE_ERR_MSG
		CALL	DO_ERROR

READ_CHAR 	ENDP


;------------------------------------------------------------------------------
; NAME:        	CLOSE_FILE
;
; PURPOSE:     	CLOSE THE FILE
;------------------------------------------------------------------------------
CLOSE_FILE	PROC 	NEAR

		MOV  	BX,FILE_HANDLE    
                MOV  	AH,03EH
                INT  	21H

;-----	DONE, RETURN

		RET

CLOSE_FILE	ENDP


;------------------------------------------------------------------------------
; NAME:        	CHAR_TO_LINE
;
; PURPOSE:     	PLACE CHARACTER IN LINE_BUFFER.  (LINE_BUFFER TO BE USED LATER 
;		 FOR PLACING A LINEFUL OF DATA IN A SPREADSHEET CELL.)  BUMP
;		 POINTER INDICATING CURRENT POSITION WITHIN LINE_BUFFER.
;
;		AFTER PUTTING THE CHARACTER IN THE BUFFER, TEST WHETHER THE
;		 BUFFER IS FULL. IF SO, DON'T BUMP THE POSITION POINTER.  (THIS
;		 HANDLES THE PROBLEM WHEN A LINE IS LONGER THAN 240 CHARS.)
;
; PARAMETERS:	FILE_CHAR   - CHARACTER TO BE INSERTED IN LINE_BUFFER
;		CUR_POINTER - POINTER TO POSITION WITHIN LINE_BUFFER
;------------------------------------------------------------------------------
CHAR_TO_LINE	PROC	NEAR

;-----	MOVE CHARACTER INTO LINE BUFFER

		MOV	AL,FILE_CHAR		
		MOV	DI,CUR_POINTER		;DI=CURRENT BUFFER POSITION
		MOV	[DI],AL

;-----	CASE: LINE BUFFER FULL (HAS 241 CHARACTERS IN IT)?

		MOV	AX,CUR_POINTER
		SUB	AX,OFFSET LINE_BUFFER

		CMP	AX,241		;YES, LEAVE WITH CUR_POINTER UNCHANGED
		JZ	CTL1

;-----	BUMP CUR_POINTER BY 1

		MOV	AX,CUR_POINTER		
		INC	AX
		MOV	CUR_POINTER,AX
CTL1:

;----	DONE, RETURN

		RET

CHAR_TO_LINE	ENDP


;------------------------------------------------------------------------------
; NAME:		LINE_TO_CELL
;
; PURPOSE:	PUT CONTENTS OF LINE_BUFFER INTO A SPREADSHEET CELL.
;
;		BEFORE PLACING DATA INTO CELL, CHECK TO SEE IF THE MAXIMUM
;		 ROW BOUNDARY (8191) HAS BEEN OVERSTEPPED.  ALSO, IF THE LINE
;		 IS ACTUALLY BLANK, DON'T PUT ANYTHING IN THE CELL.
;
;		AFTER PLACING DATA IN SPREADSHEET, BUMP THE CURRENT ROW BY 1
;		 AND RESET CUR_POINTER TO THE START OF LINE_BUFFER.
;
; PARAMETERS:	CUR_ROW	    - CURRENT SPREADSHEET ROW (ZERO BASED)
;		CUR_COL	    - CURRENT SPREADSHEET COL (ZERO BASED)
;		CUR_POINTER - POINTER TO POSITION WITHIN LINE_BUFFER
;		LINE_BUFFER - BUFFER CONTAINING LINEFUL OF DATA
;------------------------------------------------------------------------------
LINE_TO_CELL	PROC	NEAR

;-----	CASE: BEYOND MAXIMUM ROW BOUNDARY OF 8191 (ZERO BASED)?

		CMP	CUR_ROW,MAXROW+1	;YES, DO FATAL ERROR
		JZ	LTC1

;-----	CASE: BLANK LINE (I.E.,FIRST CHAR IS HARD CR)?

		MOV	AL,LINE_BUFFER+1
		CMP	AL,14H			;YES, GO BUMP ROW NUMBER
		JZ	LTC2			;     AND LEAVE

;-----	PLACE THE CONTENTS OF LINE BUFFER IN SPREADSHEET CELL

		MOV	AH,FMT_DEFAULT		;AH=DEFAULT CELL FORMAT
		OR	AH,080H			;SET PROTECTION BIT
		MOV	AL,T_LABEL		;AL=CELL TYPE IS LABEL
		MOV	CX,CUR_COL		;CX=CURRENT COLUMN
		MOV	DX,CUR_ROW		;DX=CURRENT ROW
		LEA	SI,LINE_BUFFER		;SI=OFFSET TO LINE_BUFFER

		CALL	IN_CELL

LTC2:
;-----	BUMP THE CURRENT ROW NUMBER

		MOV	AX,CUR_ROW
		INC	AX
		MOV	CUR_ROW,AX

;-----	DONE, RETURN

		RET

LTC1:
;------------------------------------------------------------------------------
; ERROR: CURRENT ROW IS BEYOND MAXIMUM ROW IN SPREADSHEET
;------------------------------------------------------------------------------

;-----	DO A FATAL ERROR

		LEA	SI,ROW_ERR_MSG
		CALL	DO_ERROR

LINE_TO_CELL	ENDP


;------------------------------------------------------------------------------
; NAME:		XDV_POLL
;
; PURPOSE:	CALLED WHEN WAITING FOR A KEY TO BE PRESSED
;------------------------------------------------------------------------------
XDV_POLL	PROC	NEAR
		RET
XDV_POLL	ENDP


;------------------------------------------------------------------------------
; NAME:		XDV_ABORT
;
; PURPOSE:	CALLED IF THERE IS A FATAL ERROR
;------------------------------------------------------------------------------
XDV_ABORT	PROC	NEAR

;-----	STORE SEGMENT, SET SEGMENT TO NEW

		PUSH	DS
		MOV	AX,CS
		MOV	DS,AX
		MOV	ES,AX

;-----	CHECK: ADD-IN ACTIVE?

		CMP	ACTIVE_FLAG,TRUE	
		JNZ	XA1			;NO, EXIT 

		MOV	ACTIVE_FLAG,FALSE	;YES, RESET ACTIVE_FLAG

;-----	CHECK: FILE OPEN?

		CMP	FILE_FLAG,TRUE
		JNZ 	XA1			;NO, EXIT	

		MOV	FILE_FLAG,FALSE		;YES, RESET OPEN FILE FLAG
 		CALL	CLOSE_FILE		;     CLOSE FILE

XA1:
;----- 	DONE, RETURN

		POP	DS
		RET

XDV_ABORT	ENDP


;------------------------------------------------------------------------------
; NAME:		XDV_HELP
;
; PURPOSE:	CALLED IF HELP KEY IS PRESSED DURING ADD-IN EXECUTION
;------------------------------------------------------------------------------
XDV_HELP	PROC	NEAR

;-----	STORE SEGMENT, SET SEGMENT TO NEW

		PUSH	DS
		MOV	AX,CS
		MOV	DS,AX
		MOV	ES,AX

;-----	ERASE 21 CONTIGUOUS LINES FROM THE SCREEN 

		MOV	AX,3		;BEGIN AT LINE 3 (ZERO BASED)
		MOV	CX,21		;ERASE 21 LINES (INCLUDING LINE 3)
		CALL	DO_ERASE_LINES

;-----	TURN OFF THE HARDWARE CURSOR

		CALL	DO_HIDEMARK

;-----	DISPLAY HELP SCREEN

		CALL	DISPLAY_HELP

;-----	WAIT FOR USER TO PRESS A CHARACTER

		CALL	GET_CHAR

;-----	ERASE 21 CONTIGUOUS LINES FROM THE SCREEN 

		MOV	AX,3		;BEGIN AT LINE 3 (ZERO BASED)
		MOV	CX,21		;ERASE 21 LINES (INCLUDING LINE 3)
		CALL	DO_ERASE_LINES

;-----	REDISPLAY THE 1-2-3 SCREEN

		CALL	REDISPLAY

;----- 	DONE, RETURN

		POP	DS
		RET

XDV_HELP	ENDP


;------------------------------------------------------------------------------
; NAME:		DISPLAY_HELP
;
; PURSPOSE:	DISPLAY THE HELP SCREEN 
;------------------------------------------------------------------------------
DISPLAY_HELP	PROC	NEAR

;-----	PUT UP SOLID LINE

		MOV	CX,0		;START AT COL 0 (ZERO BASED) 
		MOV	DX,3		;	  ROW 3 (ZERO BASED)
		CALL	DO_SETCUR	;SET THE IMAGINARY OUTPUT CURSOR

		MOV	AH,AT_NORMAL	;TEXT DISPLAY ATTRIBUTE

		LEA	SI,HELP_LINE
		CALL	DO_WRITE

;-----	PUT UP HELP SCREEN TITLE

		INC	DX		
		CALL	DO_SETCUR
		
		LEA	SI,HELP1
		CALL	DO_WRITE

;-----	FIRST PARAGRAPH

		INC	DX		
		INC	DX
		CALL	DO_SETCUR

		LEA	SI,HELP2
		CALL	DO_WRITE

		INC	DX
		CALL	DO_SETCUR

		LEA	SI,HELP3
		CALL	DO_WRITE

		INC	DX
		CALL	DO_SETCUR

		LEA	SI,HELP4
		CALL	DO_WRITE

		INC	DX
		CALL	DO_SETCUR

		LEA	SI,HELP5
		CALL	DO_WRITE

		INC	DX
		CALL	DO_SETCUR

		LEA	SI,HELP6
		CALL	DO_WRITE

		INC	DX
		CALL	DO_SETCUR

		LEA	SI,HELP7
		CALL	DO_WRITE

;-----	SECOND PARAGRAPH

		INC	DX		
		INC	DX
		CALL	DO_SETCUR

		LEA	SI,HELP8
		CALL	DO_WRITE

		INC	DX
		CALL	DO_SETCUR

		LEA	SI,HELP9
		CALL	DO_WRITE

		INC	DX
		CALL	DO_SETCUR

		LEA	SI,HELP10
		CALL	DO_WRITE

		INC	DX
		CALL	DO_SETCUR

		LEA	SI,HELP11
		CALL	DO_WRITE

;-----	THIRD PARAGRAPH

		INC	DX		
		INC	DX
		CALL	DO_SETCUR

		LEA	SI,HELP12
		CALL	DO_WRITE

		INC	DX		
		CALL	DO_SETCUR

		LEA	SI,HELP13
		CALL	DO_WRITE

;-----	ENDING SOLID LINE

		INC	DX
		CALL	DO_SETCUR

		LEA	SI,HELP_LINE
		CALL	DO_WRITE

;-----	SYNC THE VIDEO TO MAKE SURE THE HELP SCREEN IS DISPLAYED

		CALL	DO_VIDEOSYNC

		RET

DISPLAY_HELP	ENDP


;------------------------------------------------------------------------------
; NAME:		XDV_ATTACH
;
; PURPOSE:	CALLED WHEN ADD-IN IS FIRST ATTACHED
;------------------------------------------------------------------------------
XDV_ATTACH	PROC	NEAR
		RET
XDV_ATTACH	ENDP


;------------------------------------------------------------------------------
; HAL ENTRY POINTS
;------------------------------------------------------------------------------
XDV_UNDO_SAVE	PROC	NEAR
		PUSH	DS
		MOV	AX,CS
		MOV	DS,AX

		POP	DS
		RET
XDV_UNDO_SAVE	ENDP

XDV_UNDO_OKAY	PROC	NEAR
		PUSH	DS
		MOV	AX,CS
		MOV	DS,AX

		POP	DS
		RET
XDV_UNDO_OKAY	ENDP

XDV_UNDO_SWAP	PROC	NEAR
		PUSH	DS
		MOV	AX,CS
		MOV	DS,AX

		POP	DS
		RET
XDV_UNDO_SWAP	ENDP

XDV_SYSINFO	PROC	NEAR
		RET
XDV_SYSINFO	ENDP


;------------------------------------------------------------------------------
; OTHER ENTRY POINTS
;------------------------------------------------------------------------------
XDV_TERMINATE	PROC	NEAR
		RET
XDV_TERMINATE	ENDP


XDV_SAVE	PROC	NEAR
		RET
XDV_SAVE	ENDP


XDV_DETACH	PROC	NEAR
		RET
XDV_DETACH	ENDP


XDV_ADJUST	PROC	NEAR
		RET
XDV_ADJUST	ENDP


XDV_CLEAR	PROC	NEAR
		RET
XDV_CLEAR	ENDP


XDV_RETRIEVE	PROC	NEAR
		RET
XDV_RETRIEVE	ENDP


XDV_IMPORT	PROC	NEAR
		RET
XDV_IMPORT	ENDP


XDV_EXPORT	PROC	NEAR
		RET
XDV_EXPORT	ENDP



;-----	END OF THE SEGMENT
CODE		ENDS
		END


